import Path from "node:path";
import type { BunFile } from "bun";
import { CLIError } from "./error";
import type { CLIOptions } from "./options";

/**
 * Check if a given path is a directory
 *
 * @param path - The path to check
 * @returns A promise that resolves to true if the path is a directory, false otherwise
 */
export async function isDirectory(path: string): Promise<boolean> {
  const file = Bun.file(path);
  const stat = await file.stat();
  return stat.isDirectory();
}

/**
 * Given a file path, attempt to read the text of the file
 * @param filePath - The path to the file to read
 * @returns The text content of the file
 *
 * @throws CLIError if the file does not exist
 */
export async function readFile(filePath: string): Promise<string> {
  const file = Bun.file(filePath);
  const exists = await file.exists();
  if (!exists) {
    throw new CLIError(`File ${filePath} does not exist.`);
  }
  return file.text();
}

/**
 * Get the output path based on the input path and options
 * @param inputPath - The path of the input file
 * @param options - The CLI options containing output directory
 * @returns The resolved output path
 */
async function getOutputPath(inputPath: string, options: CLIOptions) {
  const inputDirname = Path.dirname(inputPath);
  const inputBasename = Path.basename(inputPath);
  const outputBasename = inputBasename.replace(/\.md$/, ".html");

  if (options.outdir) {
    const dirPath = Path.relative(options.cwd, inputDirname);
    const outputPath = Path.join(options.outdir, dirPath, outputBasename);
    return Path.resolve(outputPath);
  }

  return Path.join(inputDirname, outputBasename);
}

/**
 * Get the output file based on the input path and options
 * @param inputPath - The path of the input file
 * @param options - The CLI options containing output file and directory
 * @returns The BunFile object representing the output file
 */
export async function getOutputFile(
  inputPath: string,
  options: CLIOptions,
): Promise<BunFile> {
  // --stdout option -> Force stdout
  if (options.stdout) {
    return Bun.stdout;
  }

  // Exact outfile defined -> write to file
  if (options.outfile) {
    return Bun.file(options.outfile);
  }

  // Input path with --outdir -> calculate output path
  const outputPath = await getOutputPath(inputPath, options);
  return Bun.file(outputPath);
}
