import { parseArgs } from "node:util";
import { CLIError } from "./error";

/**
 * A collection of options parsed from the command line arguments.
 */
export interface CLIOptions {
  /** A path to the output file */
  outfile: string | null;

  /** A path to the output directory */
  outdir: string | null;

  /** If true, force output to stdout */
  stdout: boolean;

  /** The path to the template file */
  templateFilePath: string | null;

  /** The path to the stylesheet file */
  stylesheetFilePath: string | null;

  /** If the default stylesheet should be included regardless of the provided stylesheet */
  includeDefaultStylesheet: boolean;

  /** If true, show help message */
  help: boolean;

  /** If provided, overrides the document title */
  title: string | null;

  /** If provided, overrides the current working directory (default: .) */
  cwd: string;
}

/**
 * Parse the command line arguments and return the options and positional arguments.
 * @returns An object containing the parsed options and positional arguments
 */
export function parseCLIArgs(): { options: CLIOptions; args: string[] } {
  const { values: flags, positionals } = parseArgs({
    args: Bun.argv.slice(2),
    options: {
      outfile: {
        type: "string",
        short: "o",
      },
      outdir: {
        type: "string",
        short: "d",
      },
      stdout: {
        type: "boolean",
      },
      template: {
        type: "string",
        short: "t",
      },
      title: {
        type: "string",
        short: "T",
      },
      stylesheet: {
        type: "string",
        short: "s",
      },
      "include-default-stylesheet": {
        type: "boolean",
        short: "S",
      },
      cwd: {
        type: "string",
        default: process.cwd(),
      },
      help: {
        type: "boolean",
        short: "h",
      },
    },
    allowPositionals: true,
  });

  // == Argument Validation ==

  // Outfile requires only one argument
  if (flags.outfile && positionals.length > 1) {
    throw new CLIError("--outfile cannot be used with multiple inputs.");
  }

  // Outfile and Outdir cannot be used together
  if (flags.outdir && flags.outfile) {
    throw new CLIError("--outdir and --outfile cannot be used together.");
  }

  return {
    options: {
      cwd: flags.cwd,
      outfile: flags.outfile ?? null,
      outdir: flags.outdir ?? null,
      stdout: flags.stdout ?? false,
      templateFilePath: flags.template ?? null,
      stylesheetFilePath: flags.stylesheet ?? null,
      includeDefaultStylesheet: flags["include-default-stylesheet"] ?? false,
      help: flags.help ?? false,
      title: flags.title ?? null,
    },
    args: positionals,
  };
}

export const USAGE = `
buildmd - Build markdown files to html with templates and metadata

Usage:
  buildmd file.md
  buildmd file.md -o output.html
  echo "some markdown" | buildmd

Options:
  --outfile, -o <file>              Output path
  --outdir,  -d <dir>               Output directory
  --stdout                          Force output to stdout
  --template, -t <file>             Template path (default: _template.html)
  --stylesheet, -s <file>           Stylesheet path (default: _style.css)
  --include-default-stylesheet, -S  Extend default CSS instead of overwriting
  --title, -T <string>              Document title override
  --help, -h                        Show this help message
`.trim();
