import Path from "node:path";
import DEFAULT_TEMPLATE from "./defaults/default-template.html" with {
  type: "text",
};
import { readFile } from "./file-util";
import type { CLIOptions } from "./options";

const DEFAULT_TEMPLATE_FILE = "_template.html";

/**
 * Renders the provided template with the given data.
 * @param template - The template string containing placeholders like %key%
 * @param data - An object containing key-value pairs to replace in the template
 * @returns The rendered string with all placeholders replaced by their corresponding values
 */
export function renderTemplate(
  template: string,
  data: Record<string, unknown>,
): string {
  let output = template;
  for (const [key, value] of Object.entries(data)) {
    output = output.replace(new RegExp(`%${key}%`, "g"), String(value));
  }
  return output;
}

/**
 * Attempt to load a custom template, falling back to a default
 * @param options - The CLI options containing the template file path
 * @returns The template string
 */
export async function loadTemplate(options: CLIOptions): Promise<string> {
  if (options.templateFilePath) {
    return readFile(options.templateFilePath);
  }

  const checkTemplateFile = Bun.file(
    Path.join(options.cwd, DEFAULT_TEMPLATE_FILE),
  );

  if (await checkTemplateFile.exists()) {
    return checkTemplateFile.text();
  }

  return DEFAULT_TEMPLATE;
}
